=begin
(c) TIG 2009
material-name=.rb adds a new Method to Sketchup's Material Class- 
  material.name="new_name"
it makes a cloned new-material from the original-material 
and gives it the 'new_name', 
if the name 'new_name' exists then it's made unique with a #N suffix, 
it gives the new-material to any entities that had the original-material, 
the original-material is deleted from the model and material-browser: 
if the current-material in the browser was the original-material then 
the current-material in the browser becomes the new_material...
v1.0 1st issue 20090624
v1.1 Temp folder fixed for Mac users.
v1.2 Any Attribute Dictionaries copied across.
v1.3 Error with attrdict==nil fixed.
=end
require 'sketchup.rb'
class Sketchup::Material
  def name=(new_name)
    model=Sketchup.active_model
    ents=model.entities; defs=model.definitions; mats=model.materials
    return nil if not new_name or new_name=="" or new_name==self.display_name
    new_name_s=new_name.to_s; new_name=new_name_s
    names=[];mats.each{|m|names.push(m.display_name)}
    num=1
    while names.include?(new_name)
      new_name=new_name_s+"#"+num.to_s; num+=1
    end#if ### so it gets a unique name
    ### get self details
    alpha=self.alpha
    color=self.color
    texture=self.texture
    attrdicts=self.attribute_dictionaries ################### v1.2
    ### make new material to match self
    newMat=mats.add(new_name)
    newMat.alpha=alpha
    newMat.color=color
    ################### v1.2
    attrdicts.each{|attrdict|
      name=attrdict.name
      attrdict.each_pair{|key,value|newMat.set_attribute(name,key,value)}
    }if attrdicts ### v1.3
    ########################################################## v1.2
    if texture
      size=texture.height
      filename=texture.filename
      if filename.include?("\\") or filename.include?("/")#PC || Mac
        if FileTest.exist?(filename)### it's image was set 'manually'
          newMat.texture=filename.tr("\\","/")
          newMat.texture.size=size
        else### was temp, so not found, re-save and use it's name
          filename=filename.tr("\\","/").split("/").last
          tdir=ENV["TEMP"]
          tdir=ENV["TMPDIR"]if not tdir ### it's a Mac v1.1
          tdir=tdir.tr("\\","/")+"/"
          filepath=tdir+filename
          group=ents.add_group
          group.material=self
          tw=Sketchup.create_texture_writer
          tw.load(group)
          tw.write(group,filepath)
          newMat.texture=filepath
          newMat.texture.size=size
          File.delete(filepath)
        end#if
      else ### it's a bound image
        tdir=ENV["TEMP"]
        tdir=ENV["TMPDIR"]if not tdir ### it's a Mac v1.1
        tdir=tdir.tr("\\","/")+"/"
        filepath=tdir+filename
        group=ents.add_group
        group.material=self
        tw=Sketchup.create_texture_writer
        tw.load(group)
        tw.write(group,filepath)
        newMat.texture=filepath
        newMat.texture.size=size
        File.delete(filepath)
      end#if
    end#if
    ### tidy up the materials
    ents.each{|e|e.material=newMat if e.respond_to?(:material)and e.material==self
      e.back_material=newMat if e.kind_of?(Sketchup::Face)and e.back_material==self}
    defs.each{|d|d.entities.each{|e|e.material=newMat if e.respond_to?(:material)and e.material==self
      e.back_material=newMat if e.kind_of?(Sketchup::Face)and e.back_material==self}}
    group=ents.add_group();gents=group.entities ### temporarily use other materials
    temp=gents.add_group()
    (mats.to_a-[self]).each{|m|tc=temp.copy;tc.material=m}
    mats.current=newMat if mats.current==self ### ensure self is not current material
    mats.purge_unused ### purge self from browser
    group.erase! ### erase! the temporary material user, use set as was.
  end#def
end#class
